library(loadeR)
library(transformeR)
library(ggplot2)
library(RColorBrewer)
library(maps)
library(gridExtra)

rm(list=ls()) ; graphics.off() ; gc()

#Directory information
setwd("~/Documents/ETCCDI_eobs/")
OUTPUTS <- "OUTPUTS/"

#####################################
#Time series CPs
#Load CPs information
load("../WardCluster_Dendrogram_D4_50km.Rdata")
k=8

calendario <- seq(as.Date("1950-01-01"),as.Date("2021-12-31"),by="day")
fechas <- calendario[format(calendario,"%m-%d") != "02-29"]
seasons <- list(Summer=c(6,7,8),Autumn=c(9,10,11),Winter=c(12,1,2),Spring=c(3,4,5))

vector_cluster <- cutree(ward_clustering,k)[which(format(calendario,"%Y")%in%c(1950:2021))]
cp <- loadGridData(dataset = "../eobs_tasmax_spain.nc",lonLim = c(-1,1),latLim = c(35,36),var="tasmax")
cp <- aggregateGrid(cp,aggr.spatial = list(FUN="mean"))
for(l in 1:length(cp$Data)){
  cp$Data[l] <- vector_cluster[l]
}

##################
#Klee diagram
calendario <- seq(as.Date(paste(getYearsAsINDEX(cp)[1],"-01-01",sep="")),as.Date(paste(getYearsAsINDEX(cp)[length(getYearsAsINDEX(cp))],"-12-31",sep="")),by="day")
fechas <- calendario[format(calendario,"%m-%d") != "02-29"]

Year <- as.numeric(format(fechas,"%Y"))
Month <- as.numeric(format(fechas,"%m"))
Day <- rep(c(1:365),length(unique(Year)))

data <- data.frame(Year=Year,Month=Month,Day=Day,CP=cp$Data[format(calendario,"%m-%d") != "02-29"])
data$CP <- factor(data$CP,levels=c(1:k))

##################
#Klee diagram
palette <- brewer.pal(n=8,name = "Set1")

calendario <- seq(as.Date(paste(getYearsAsINDEX(cp)[1],"-01-01",sep="")),as.Date(paste(getYearsAsINDEX(cp)[length(getYearsAsINDEX(cp))],"-12-31",sep="")),by="day")
fechas <- calendario[format(calendario,"%m-%d") != "02-29"]

Year <- as.numeric(format(fechas,"%Y"))
Month <- as.numeric(format(fechas,"%m"))
Day <- rep(c(1:365),length(unique(Year)))

data <- data.frame(Year=Year,Month=Month,Day=Day,CP=cp$Data[format(calendario,"%m-%d") != "02-29"])
data$CP <- factor(data$CP,levels=c(1:k))

g <- ggplot(data=data,aes(x=Year,y=Day,fill=CP)) + geom_tile() +theme_bw()+theme(axis.text.x = element_text(angle = 90, vjust = 0.5, hjust=1))+
  scale_x_continuous(n.breaks = 8)+ coord_cartesian(expand=F)+scale_y_continuous(labels = c("J","F","M","A","M","J","J","A","S","O","N","D"),breaks = seq(15,365,by=30))+ scale_fill_manual(values=palette[1:k],aesthetics=c("color","fill"))

png(filename = paste(OUTPUTS,"CPs_Klee_D4k8_cols.png",sep=""),res=600,width=6000,height=3000)
print(ggplot(data=data,aes(x=Year,y=Day,fill=CP)) + geom_tile() +theme_bw()+theme(axis.text.x = element_text(angle = 90, vjust = 0.5, hjust=1))+
        scale_x_continuous(n.breaks = 8)+ coord_cartesian(expand=F)+scale_y_continuous(labels = c("J","F","M","A","M","J","J","A","S","O","N","D"),breaks = seq(15,365,by=30))+ scale_fill_manual(values=palette[1:k]),aesthetics=c("color","fill"))
# ggtitle(paste("CPs relative frequency","D2 k1",sep=" ")) + geom_smooth(method=lm,se=F,lwd=0.75))
dev.off()

#############
#Decadal

decades <- list("1950-1959"=c(1950:1959),
                "1960-1969"=c(1960:1969),
                "1970-1979"=c(1970:1979),
                "1980-1989"=c(1980:1989),
                "1990-1999"=c(1990:1999),
                "2000-2010"=c(2000:2010),
                "2010-2020"= c(2010:2020))

value <- list()

#Monthly
for(d in 1:length(decades)){
  value[[d]] <- matrix(NA,12,length(unique(data$CP)))
  decada <- subset(data,Year%in%decades[[d]])
  decada$CP <- as.numeric(decada$CP)
  for(k in 1:length(unique(decada$CP))){
    for(m in 1:12){
    value[[d]][m,k] <-  length(which(decada$CP==k & decada$Month==m))
    }
  }
}

datitos <- data.frame(Month=rep(1:12,length(decades)*length(unique(data$CP))),
                      Frequency=unlist(value),
                      CP=rep(rep(1:length(unique(data$CP)),each=12,length(decades))),
                      Decade=rep(names(decades),each=length(as.numeric(value[[1]]))))
datitos$Decade <- as.factor(datitos$Decade)

p <- ggplot(data=datitos,aes(x=Month,y=Frequency,colour=Decade)) + geom_line()+facet_wrap(~CP,nrow=2,scales = "free_y") + theme_bw()+
 coord_flip()+ scale_color_manual(values=brewer.pal(7,"Blues")) + scale_x_continuous(breaks=1:12)

png(filename = paste(OUTPUTS,"CPs_DecadalKlee_D4k8_cols.png",sep=""),res=600,width=6000,height=3000)
print(p)
dev.off()


png(filename = paste(OUTPUTS,"KleeD4.8both.png",sep=""),res=600,width=6000,height=6000)
print(grid.arrange(g,p,nrow=2))
dev.off()

