library(loadeR)
library(transformeR)
library(ggplot2)
library(RColorBrewer)
library(maps)
#library(climate4R.value)
library(Kendall)
library(trend)

rm(list=ls()) ; graphics.off() ; gc()

#Directory information
setwd("/esarchive/scratch/molmo/ETCCDI_aemet-5km/ETCCDI_aemet-5km/")
OUTPUTS <- "../OUTPUTS/"

#Useful information
ETCCDI_index <- c("Rx1day","Rx5day","TN10p","TN90p","TNn","TNx","TX10p","TX90p","TXn","TXx","CDD","CWD","CSDI",
                  "PRCPTOT","TR","R10mm","R20mm","R95pTOT","WSDI")
seasons <- list(Summer=c(6,7,8),Autumn=c(9,10,11),Winter=c(12,1,2),Spring=c(3,4,5))

calendario <- seq(as.Date("1950-01-01"),as.Date("2021-12-31"),by="day")
fechas <- calendario[format(calendario,"%m-%d") != "02-29"]

for(ind in 1:length(ETCCDI_index)){
  #Load data
  files <- list.files(pattern = paste(ETCCDI_index[ind],"_aemet-5km",sep=""))
  data <- list() 
  
  for(i in 1:length(files)){
    di <- dataInventory(dataset = files[i])
    nc <- loadGridData(dataset=files[i],var=names(di),years=c(1951:2020))
    
    #Aggregation of annual/seasonal indices (climatology: time-series)
    if(length(nc$Dates$start)<=72){
      seasons <- list("Annual"=c(1:12))
      prueba <- climatology(nc,clim.fun=list(FUN="trend.1D",dates=nc$Dates$start))
      sig <-  climatology(nc,clim.fun=list(FUN="trend.1D",dates=nc$Dates$start,
                                           method="kendall",return.pvalue=T))
      
      #Map of trends
      data[[1]] <- data.frame(Trend=10*as.numeric(redim(prueba,drop=T)$Data)/length(nc$Dates$start),
                              Season=rep("Annual",length(as.numeric(redim(prueba,drop=T)$Data))),
                              Sign=as.numeric(redim(sig,drop=T)$Data),
                              Index=ETCCDI_index[ind],
                              Latitude=rep(prueba$xyCoords$y,length(prueba$xyCoords$x)),
                              Longitude=rep(prueba$xyCoords$x,each=length(prueba$xyCoords$y)))
  
    }
    if(length(nc$Dates$start)>72){
      for(j in 1:length(seasons)){
        nc2 <- aggregateGrid(nc,aggr.s = list(FUN=list("mean",na.rm=T),season=seasons[[j]]))
        prueba <- climatology(nc2,clim.fun=list(FUN="trend.1D",dates=nc2$Dates$start))
        sig <-  climatology(nc2,clim.fun=list(FUN="trend.1D",dates=nc2$Dates$start,
                                             method="kendall",return.pvalue=T))
        
        #Map of trends
        data[[j]] <- data.frame(Trend=10*as.numeric(redim(prueba,drop=T)$Data)/length(nc2$Dates$start),
                                Season=rep(names(seasons)[j],length(as.numeric(redim(prueba,drop=T)$Data))),
                                Sign=as.numeric(redim(sig,drop=T)$Data),
                                Index=ETCCDI_index[ind],
                                Latitude=rep(prueba$xyCoords$y,length(prueba$xyCoords$x)),
                                Longitude=rep(prueba$xyCoords$x,each=length(prueba$xyCoords$y)))
      }}
    
  }
  
  #Organize for ggplot plotting and saving
  datos <- do.call(rbind,data)
  datos$Season <- factor(datos$Season,levels=names(seasons))
  
  #Plot
  paleta = brewer.pal(9,"BrBG")
  datos$Unit <- NA
  datos$Unit[which(datos$Sign<=0.05)] <- 1
  datos$Unit[which(datos$Sign>0.05)] <- 0
  
  if(ETCCDI_index[ind]%in%c("Rx1day","Rx5day","PRCPTOT","R10mm","R20mm","R95pTOT")){
    paleta=brewer.pal(11,"BrBG");units="mm";range=c(-0.12,0.12)}
  if(ETCCDI_index[ind]%in%c("R10mm","R20mm","CWD")){
    paleta=brewer.pal(11,"BrBG"); units="days"; range=c(-0.12,0.12)}
  if(ETCCDI_index[ind]%in%c("CDD")){
    paleta=rev(brewer.pal(11,"BrBG")); units="days";range=c(-0.12,0.12)}
  
  if(ETCCDI_index[ind]%in%c("TN10p","TN90p","TX10p","TX90p","CSDI","WSDI","TR")){
    paleta=rev(brewer.pal(11,"RdYlBu"));units="days";range=c(-0.12,0.12)}
  if(ETCCDI_index[ind]%in%c("TNn" ,"TNx","TXn","TXx")){
    paleta=rev(brewer.pal(11,"RdYlBu"));units="°C";range=c(-0.12,0.12)}
  
  #tengo que hacer un ploteo que sea solo del área significativa para entender bien los contornos
  
  
  png(filename = paste(OUTPUTS,"Trend_GridCell","_",ETCCDI_index[ind],".png",sep=""),res=300,width=3000,height=1500)
  
  print(
    ggplot(data=datos,aes(x=Longitude,y=Latitude,fill=Trend)) + geom_tile() + theme_bw() + ggtitle(ETCCDI_index[ind])+
    scale_fill_gradientn(name=paste(units,"decade ",sep="/"),colours=paleta,limits=range,na.value = "white")+
    coord_quickmap(xlim=c(min(datos$Longitude),max(datos$Longitude)),
                   ylim=c(min(datos$Latitude),max(datos$Latitude)),expand=F)+ borders() + facet_wrap(~Season,nrow=1)+
      geom_contour(data=datos,aes(z=Unit),colour="black")#,aes(label="*",fontface="bold",size=0.01))
  )  
  dev.off()
  
  png(filename = paste(OUTPUTS,"Trend_GridCell","_",ETCCDI_index[ind],"_Sig.png",sep=""),res=300,width=3000,height=1500)
  
  print(
    ggplot(data=subset(datos,Unit==1),aes(x=Longitude,y=Latitude,fill=Trend)) + geom_tile() + theme_bw() + ggtitle(ETCCDI_index[ind])+
      scale_fill_gradientn(name=paste(units,"decade ",sep="/"),colours=paleta,limits=range,na.value = "white")+
      coord_quickmap(xlim=c(min(datos$Longitude),max(datos$Longitude)),
                     ylim=c(min(datos$Latitude),max(datos$Latitude)),expand=F)+ borders() + facet_wrap(~Season,nrow=1)+
      geom_contour(data=subset(datos,Unit==1),aes(z=Unit),colour="black")#,aes(label="*",fontface="bold",size=0.01))
  )  
  dev.off()
}


#esto es de heatmap
data <- do.call("rbind",mix)
data <- data[-which(data$Index%in%c("R95pTOT","R10mm","PRCPTOT")),]
data$Index <- factor(data$Index,levels=rev(c("TN10p","TN90p","TNn","TNx","TR","CSDI","TX10p","TX90p","TXn","TXx","WSDI","Rx1day","Rx5day","CDD","CWD",
                                             "R20mm")))
png(filename = paste(OUTPUTS,"Trends_aemet-5km_ETCCDI.png",sep=""),res=600,width=3000,height=4000)
print(ggplot(data=data,aes(x=Season,y=Index,fill=Trend)) + geom_tile() + facet_wrap(~Region,nrow = 2) +
        theme_bw() + ggtitle("AEMET-5km") + 
        scale_color_gradientn("Trend",colours = rev(brewer.pal(n=11,name="RdBu")),limits=c(-0.4,0.4),
                              aesthetics = "fill",na.value = "white") + 
        geom_text(data=subset(data,Sign<=0.05),aes(label="*",fontface="bold")))  
dev.off()


